<?php
// This file is part of the Studyplan plugin for Moodle
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.
/**
 *
 * @package    local_treestudyplan
 * @copyright  2023 P.M. Kuipers
 * @license    https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

require_once("$CFG->libdir/formslib.php");
require_once("$CFG->dirroot/local/treestudyplan/lib.php");

class reportinvite_form extends moodleform {
    // Add elements to form.

    public function definition() {
        global $CFG;

        $mform = $this->_form; // Don't forget the underscore!

        $mform->addElement('hidden', 'add', 0);
        $mform->setType('add', PARAM_ALPHANUM);

        $mform->addElement('hidden', 'update', 0);
        $mform->setType('update', PARAM_INT);

        $mform->addElement('text', 'name', get_string('invite_name', 'local_treestudyplan'), array('size' => 50));
        $mform->setType('name', PARAM_NOTAGS);                   // Set type of element.
        $mform->setDefault('name', '');        // Default value.
        $mform->addRule('name', get_string('required'), 'required', null, 'client');

        $mform->addElement('text', 'email', get_string('invite_email', 'local_treestudyplan'), array('size' => 20));
        $mform->setType('email', PARAM_NOTAGS);                   // Set type of element.
        $mform->setDefault('email', '');        // Default value.
        $mform->addRule('email', get_string('required'), 'required', null, 'client');
        $mform->addRule('email', get_string('email'), 'email', null, 'client');

        $mform->addElement('static', get_string('invite_email', 'local_treestudyplan') );

        $this->add_action_buttons();
    }
    // Custom validation should be added here.
    public function validation($data, $files) {
        return array();
    }

    public function get_data() {
        global $DB, $USER;

        $data = parent::get_data();
        if ($data != null) {
            if (empty($data->user_id)) {
                $data->user_id = $USER->id;
            }

            if (empty($data->update)) {
                $date = new DateTime("now", core_date::get_user_timezone_object());
                $date->setTime(0, 0, 0);

                $data->idate = $date->getTimeStamp();
            }

            if (empty($data->update)) {
                // Create a new random key for the invite.
                do {
                    $length = 20;
                    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
                    $characterslength  = strlen($characters);
                    $randomkey = '';
                    for ($i = 0; $i < $length; $i++) {
                        $randomkey .= $characters[rand(0, $characterslength - 1)];
                    }

                    // Double check that the key is unique before inserting.
                } while ($DB->record_exists_select("local_treestudyplan_invit",
                                                  $DB->sql_compare_text("invitekey"). " = " . $DB->sql_compare_text(":invitekey"),
                                                  ['invitekey' => $randomkey]));

                $data->invitekey = $randomkey;

            }
        }
        return $data;
    }
}
